@@ -1,6 +1,8 @@
 // Generalized callback interface - shared between global AI and group AI
 #include "StdAfx.h"
 #include "FileSystem/FileHandler.h"
+#include "FileSystem/FileSystem.h"
+#include "FileSystem/FileSystemHandler.h"
 #include "Game/Camera/CameraController.h"
 #include "Game/Camera.h"
 #include "Game/CameraHandler.h"
@@ -18,7 +20,6 @@
 #include "NetProtocol.h"
 #include "ConfigHandler.h"
 #include "Platform/errorhandler.h"
-#include "FileSystem/FileSystem.h"
 #include "Rendering/InMapDraw.h"
 #include "Rendering/UnitModels/3DModel.h"
 #include "Rendering/UnitModels/UnitDrawer.h"
@@ -41,9 +42,10 @@
 #include "Sim/Weapons/WeaponDefHandler.h"
 #include "Sim/Weapons/Weapon.h"
 #include "AICheats.h"
-#include "GlobalAICallback.h"
-#include "SkirmishAIWrapper.h"
-#include "EngineOutHandler.h"
+#include "ExternalAI/GlobalAICallback.h"
+#include "ExternalAI/SkirmishAIHandler.h"
+#include "ExternalAI/SkirmishAIWrapper.h"
+#include "ExternalAI/EngineOutHandler.h"
 #include "Sim/Units/Groups/Group.h"
 #include "Sim/Units/Groups/GroupHandler.h"
 #include "LogOutput.h"
@@ -69,17 +71,17 @@ CAICallback::~CAICallback(void)
 
 void CAICallback::SendStartPos(bool ready, float3 startPos)
 {
-	if(startPos.z<gameSetup->allyStartingData[gu->myAllyTeam].startRectTop *gs->mapy*8)
-		startPos.z=gameSetup->allyStartingData[gu->myAllyTeam].startRectTop*gs->mapy*8;
+	if (startPos.z < gameSetup->allyStartingData[gu->myAllyTeam].startRectTop * gs->mapy * SQUARE_SIZE)
+		startPos.z = gameSetup->allyStartingData[gu->myAllyTeam].startRectTop * gs->mapy * SQUARE_SIZE;
 
-	if(startPos.z>gameSetup->allyStartingData[gu->myAllyTeam].startRectBottom*gs->mapy*8)
-		startPos.z=gameSetup->allyStartingData[gu->myAllyTeam].startRectBottom*gs->mapy*8;
+	if (startPos.z > gameSetup->allyStartingData[gu->myAllyTeam].startRectBottom * gs->mapy * SQUARE_SIZE)
+		startPos.z = gameSetup->allyStartingData[gu->myAllyTeam].startRectBottom * gs->mapy * SQUARE_SIZE;
 
-	if(startPos.x<gameSetup->allyStartingData[gu->myAllyTeam].startRectLeft*gs->mapx*8)
-		startPos.x=gameSetup->allyStartingData[gu->myAllyTeam].startRectLeft*gs->mapx*8;
+	if (startPos.x < gameSetup->allyStartingData[gu->myAllyTeam].startRectLeft * gs->mapx * SQUARE_SIZE)
+		startPos.x = gameSetup->allyStartingData[gu->myAllyTeam].startRectLeft * gs->mapx * SQUARE_SIZE;
 
-	if(startPos.x>gameSetup->allyStartingData[gu->myAllyTeam].startRectRight*gs->mapx*8)
-		startPos.x=gameSetup->allyStartingData[gu->myAllyTeam].startRectRight*gs->mapx*8;
+	if (startPos.x > gameSetup->allyStartingData[gu->myAllyTeam].startRectRight * gs->mapx * SQUARE_SIZE)
+		startPos.x = gameSetup->allyStartingData[gu->myAllyTeam].startRectRight * gs->mapx * SQUARE_SIZE;
 
 	unsigned char readyness = ready? 1: 0;
 	net->Send(CBaseNetProtocol::Get().SendStartPos(gu->myPlayerNum, team, readyness, startPos.x, startPos.y, startPos.z));
@@ -87,7 +89,10 @@ void CAICallback::SendStartPos(bool ready, float3 startPos)
 
 void CAICallback::SendTextMsg(const char* text, int zone)
 {
-	logOutput.Print("SkirmishAI%i: %s", team, text);
+	const CSkirmishAIHandler::ids_t& teamAIs = skirmishAIHandler.GetSkirmishAIsInTeam(this->team);
+	const SkirmishAIData* aiData = skirmishAIHandler.GetSkirmishAI(*(teamAIs.begin())); // better way?
+
+	logOutput.Print("<SkirmishAI: %s %s (team %d)>: %s", aiData->shortName.c_str(), aiData->version.c_str(), team, text);
 }
 
 void CAICallback::SetLastMsgPos(float3 pos)
@@ -187,20 +192,7 @@ bool CAICallback::PosInCamera(float3 pos, float radius)
 // (still completely insecure ofcourse, but it filters out the easiest way of cheating)
 void CAICallback::verify()
 {
-//TODO: add checks
-// the old check (see code below) checks for something which is not possible
-// anymore with the C AI interface
-//	const CSkirmishAIWrapper* skirmishAI = eoh->GetSkirmishAI(team);
-//	if (
-//			skirmishAI
-//			&& (((group
-//					&& group->handler != /*skirmishAI->gh*/grouphandlers[skirmishAI->GetTeamId()]
-//					/*&& group->handler != grouphandler*/)
-//			|| skirmishAI->GetTeamId() != team))) {
-//		handleerror (0, "AI has modified spring components(possible cheat)",
-//				"Spring is closing:", MBF_OK | MBF_EXCL);
-//		exit (-1);
-//	}
+	//TODO: add checks
 }
 
 int CAICallback::GetCurrentFrame()
@@ -694,6 +686,7 @@ bool CAICallback::IsUnitNeutral(int unitId) {
 
 int CAICallback::InitPath(float3 start, float3 end, int pathType)
 {
+	assert(((size_t)pathType) < moveinfo->moveData.size());
 	return pathManager->RequestPath(moveinfo->moveData.at(pathType), start, end);
 }
 
@@ -1232,11 +1225,9 @@ int CAICallback::GetFeatures(int* featureIds, int featureIds_sizeMax)
 		const CFeature* f = *it;
 		assert(f);
 
-		if (!f->IsInLosForAllyTeam(allyteam)) {
-			continue;
+		if (f->IsInLosForAllyTeam(allyteam)) {
+			featureIds[featureIds_size++] = f->id;
 		}
-
-		featureIds[featureIds_size++] = f->id;
 	}
 
 	return featureIds_size;
@@ -1255,11 +1246,9 @@ int CAICallback::GetFeatures(int* featureIds, int featureIds_sizeMax, const floa
 		const CFeature* f = *it;
 		assert(f);
 
-		if (!f->IsInLosForAllyTeam(allyteam)) {
-			continue;
+		if (f->IsInLosForAllyTeam(allyteam)) {
+			featureIds[featureIds_size++] = f->id;
 		}
-
-		featureIds[featureIds_size++] = f->id;
 	}
 
 	return featureIds_size;
@@ -1271,11 +1260,9 @@ const FeatureDef* CAICallback::GetFeatureDef(int featureId)
 
 	verify();
 
-	const CFeatureSet& fset = featureHandler->GetActiveFeatures();
-	const CFeatureSet::const_iterator it = fset.find(featureId);
+	const CFeature* f = featureHandler->GetFeature(featureId);
 
-	if (it != fset.end()) {
-		const CFeature* f = *it;
+	if (f) {
 		const int allyteam = teamHandler->AllyTeam(team);
 		if (f->IsInLosForAllyTeam(allyteam)) {
 			featureDef = f->def;
@@ -1295,11 +1282,9 @@ float CAICallback::GetFeatureHealth(int featureId)
 
 	verify();
 
-	const CFeatureSet& fset = featureHandler->GetActiveFeatures();
-	const CFeatureSet::const_iterator it = fset.find(featureId);
+	const CFeature* f = featureHandler->GetFeature(featureId);
 
-	if (it != fset.end()) {
-		const CFeature *f = *it;
+	if (f) {
 		const int allyteam = teamHandler->AllyTeam(team);
 		if (f->IsInLosForAllyTeam(allyteam)) {
 			health = f->health;
@@ -1315,11 +1300,9 @@ float CAICallback::GetFeatureReclaimLeft(int featureId)
 
 	verify();
 
-	const CFeatureSet& fset = featureHandler->GetActiveFeatures();
-	const CFeatureSet::const_iterator it = fset.find(featureId);
+	const CFeature* f = featureHandler->GetFeature(featureId);
 
-	if (it != fset.end()) {
-		const CFeature *f = *it;
+	if (f) {
 		const int allyteam = teamHandler->AllyTeam(team);
 		if (f->IsInLosForAllyTeam(allyteam)) {
 			return f->reclaimLeft;
@@ -1335,11 +1318,9 @@ float3 CAICallback::GetFeaturePos(int featureId)
 
 	verify();
 
-	const CFeatureSet& fset = featureHandler->GetActiveFeatures();
-	const CFeatureSet::const_iterator it = fset.find(featureId);
+	const CFeature* f = featureHandler->GetFeature(featureId);
 
-	if (it != fset.end()) {
-		const CFeature *f = *it;
+	if (f) {
 		const int allyteam = teamHandler->AllyTeam(team);
 		if (f->IsInLosForAllyTeam(allyteam)) {
 			pos = f->pos;
@@ -1393,7 +1374,7 @@ bool CAICallback::GetValue(int id, void *data)
 			std::string f((char*) data);
 			f = filesystem.LocateFile(f);
 			strcpy((char*) data, f.c_str());
-			return true;
+			return FileSystemHandler::IsReadableFile(f);
 		}case AIVAL_LOCATE_FILE_W:{
 			std::string f((char*) data);
 			std::string f_abs = filesystem.LocateFile(f, FileSystem::WRITE | FileSystem::CREATE_DIRS);
@@ -1680,7 +1661,7 @@ int CAICallback::GetMapPoints(PointMarker* pm, int pm_sizeMax, bool includeAllie
 		return pm_size;
 	}*/
 
-	std::list<unsigned char*> includeColors;
+	std::list<const unsigned char*> includeColors;
 	// include out team color
 	includeColors.push_back(teamHandler->Team(team)->color);
 	// include the team colors of all our allies
@@ -1692,7 +1673,7 @@ int CAICallback::GetMapPoints(PointMarker* pm, int pm_sizeMax, bool includeAllie
 
 	std::list<CInMapDraw::MapPoint>* points = NULL;
 	std::list<CInMapDraw::MapPoint>::const_iterator point;
-	std::list<unsigned char*>::const_iterator ic;
+	std::list<const unsigned char*>::const_iterator ic;
 	for (size_t i=0; i < inMapDrawer->numQuads && pm_size < pm_sizeMax; i++) {
 		points = &(inMapDrawer->drawQuads[i].points);
 		for (point = points->begin(); point != points->end()
@@ -1725,7 +1706,7 @@ int CAICallback::GetMapLines(LineMarker* lm, int lm_sizeMax, bool includeAllies)
 		return lm_size;
 	}*/
 
-	std::list<unsigned char*> includeColors;
+	std::list<const unsigned char*> includeColors;
 	// include out team color
 	includeColors.push_back(teamHandler->Team(team)->color);
 	// include the team colors of all our allies
@@ -1737,7 +1718,7 @@ int CAICallback::GetMapLines(LineMarker* lm, int lm_sizeMax, bool includeAllies)
 
 	std::list<CInMapDraw::MapLine>* lines = NULL;
 	std::list<CInMapDraw::MapLine>::const_iterator line;
-	std::list<unsigned char*>::const_iterator ic;
+	std::list<const unsigned char*>::const_iterator ic;
 	for (size_t i=0; i < inMapDrawer->numQuads && lm_size < lm_sizeMax; i++) {
 		lines = &(inMapDrawer->drawQuads[i].lines);
 		for (line = lines->begin(); line != lines->end()
