@@ -3,14 +3,15 @@
 
 #include <cstdlib>
 
-#include "Platform/byteorder.h"
 
 PlayerBase::PlayerBase() :
 	TeamController(),
 	rank(-1),
 	spectator(false),
 	isFromDemo(false),
-	readyToStart(false)
+	readyToStart(false),
+	desynced(false),
+	cpuUsage (0.0f)
 {
 }
 
@@ -31,12 +32,3 @@ void PlayerBase::SetValue(const std::string& key, const std::string& value)
 	else
 		customValues[key] = value;
 }
-
-void PlayerStatistics::swab()
-{
-	swabTC();
-
-	mousePixels = swabdword(mousePixels);
-	mouseClicks = swabdword(mouseClicks);
-	keyPresses = swabdword(keyPresses);
-}
