@@ -858,7 +858,7 @@ void CGroundMoveType::CheckCollisionSkid(void)
 			// stop units from reaching escape velocity
 			dif /= std::max(dist, 1.f);
 
-			if (u->mass == 100000 || !u->mobility) {
+			if (!u->mobility) {
 				float impactSpeed = -owner->speed.dot(dif);
 
 				if (impactSpeed > 0) {
@@ -879,8 +879,9 @@ void CGroundMoveType::CheckCollisionSkid(void)
 					}
 				}
 			} else {
-				float part = owner->mass / (owner->mass + u->mass);
-				float impactSpeed = (u->speed - owner->speed).dot(dif);
+				// don't conserve momentum
+				float part = (owner->mass / (owner->mass + u->mass));
+				float impactSpeed = (u->speed - owner->speed).dot(dif) * 0.5f;
 
 				if (impactSpeed > 0) {
 					midPos += dif * (impactSpeed * (1 - part) * 2);
@@ -913,6 +914,7 @@ void CGroundMoveType::CheckCollisionSkid(void)
 							0, dif * -impactSpeed * (u->mass * part));
 					}
 					owner->speed *= 0.9f;
+					u->speed *= 0.9f;
 				}
 			}
 		}
@@ -1050,13 +1052,15 @@ float3 CGroundMoveType::ObstacleAvoidance(float3 desiredDir) {
 			float avoidRight = 0.0f;
 
 
+			MoveData* moveData = owner->mobility;
+			moveData->tempOwner = owner;
+
 			vector<CSolidObject*> nearbyObjects = qf->GetSolidsExact(owner->pos, speedf * 35 + 30 + owner->xsize / 2);
 			vector<CSolidObject*> objectsOnPath;
 			vector<CSolidObject*>::iterator oi;
 
 			for (oi = nearbyObjects.begin(); oi != nearbyObjects.end(); oi++) {
 				CSolidObject* o = *oi;
-				MoveData* moveData = owner->mobility;
 				CMoveMath* moveMath = moveData->moveMath;
 
 				if (moveMath->IsNonBlocking(*moveData, o)) {
@@ -1114,6 +1118,7 @@ float3 CGroundMoveType::ObstacleAvoidance(float3 desiredDir) {
 				}
 			}
 
+			moveData->tempOwner = NULL;
 
 
 			// Sum up avoidance.
@@ -1167,7 +1172,12 @@ void CGroundMoveType::UpdateHeatMap()
 	if (!pathId)
 		return;
 
+#ifndef USE_GML
+	static std::vector<int2> points;
+#else
 	std::vector<int2> points;
+#endif
+
 	pathManager->GetDetailedPathSquares(pathId, points);
 	for (std::vector<int2>::iterator it = points.begin(); it != points.end(); ++it) {
 		pathManager->SetHeatOnSquare(it->x, it->y, owner->mobility->heatProduced, owner->id);
@@ -1175,7 +1185,6 @@ void CGroundMoveType::UpdateHeatMap()
 }
 
 
-
 // Calculates an aproximation of the physical 2D-distance between given two objects.
 float CGroundMoveType::Distance2D(CSolidObject* object1, CSolidObject* object2, float marginal)
 {
@@ -1293,7 +1302,7 @@ float CGroundMoveType::BreakingDistance(float speed)
 	if (!owner->mobility->maxBreaking) {
 		return 0.0f;
 	}
-	return fabs(speed * speed / owner->mobility->maxBreaking);
+	return fabs(speed*speed / owner->mobility->maxBreaking);
 }
 
 /*
@@ -1509,7 +1518,10 @@ void CGroundMoveType::CheckCollision(void)
 
 bool CGroundMoveType::CheckColH(int x, int y1, int y2, float xmove, int squareTestX)
 {
-	const MoveData* m = owner->mobility;
+	MoveData* m = owner->mobility;
+	m->tempOwner = owner;
+
+	bool ret = false;
 
 	for (int y = y1; y <= y2; ++y) {
 		bool blocked = false;
@@ -1585,16 +1597,21 @@ bool CGroundMoveType::CheckColH(int x, int y1, int y2, float xmove, int squareTe
 			owner->pos += posDelta;
 			owner->pos.x = xmove;
 			currentSpeed *= 0.97f;
-			return true;
+			ret = true;
+			break;
 		}
 	}
 
-	return false;
+	m->tempOwner = NULL;
+	return ret;
 }
 
 bool CGroundMoveType::CheckColV(int y, int x1, int x2, float zmove, int squareTestY)
 {
-	const MoveData* m = owner->mobility;
+	MoveData* m = owner->mobility;
+	m->tempOwner = owner;
+
+	bool ret = false;
 
 	for (int x = x1; x <= x2; ++x) {
 		bool blocked = false;
@@ -1670,11 +1687,13 @@ bool CGroundMoveType::CheckColV(int y, int x1, int x2, float zmove, int squareTe
 			owner->pos += posDelta;
 			owner->pos.z = zmove;
 			currentSpeed *= 0.97f;
-			return true;
+			ret = true;
+			break;
 		}
 	}
 
-	return false;
+	m->tempOwner = NULL;
+	return ret;
 }
 
 
@@ -1846,10 +1865,10 @@ void CGroundMoveType::TestNewTerrainSquare(void)
 				for (std::vector<int2>::iterator li = lineTable[lty][ltx].begin(); li != lineTable[lty][ltx].end(); ++li) {
 					int x = (moveSquareX + li->x) * 2;
 					int y = (moveSquareY + li->y) * 2;
-					int blockMask =
+					CMoveMath* mmath = owner->unitDef->movedata->moveMath;
+					static int blockMask =
 						(CMoveMath::BLOCK_STRUCTURE | CMoveMath::BLOCK_TERRAIN |
 						CMoveMath::BLOCK_MOBILE | CMoveMath::BLOCK_MOBILE_BUSY);
-					CMoveMath* mmath = owner->unitDef->movedata->moveMath;
 
 					if ((mmath->IsBlocked(*owner->unitDef->movedata, x, y) & blockMask) ||
 						mmath->SpeedMod(*owner->unitDef->movedata, x, y) <= 0.01f) {
@@ -1940,7 +1959,7 @@ void CGroundMoveType::KeepPointingTo(float3 pos, float distance, bool aggressive
 		if (dir2 != ZeroVector) {
 			short heading =
 				GetHeadingFromVector(dir2.x, dir2.z) -
-				GetHeadingFromVector(dir1.x,dir1.z);
+				GetHeadingFromVector(dir1.x, dir1.z);
 			if (owner->heading != heading
 					&& !(owner->weapons.front()->TryTarget(
 					mainHeadingPos, true, 0))) {
@@ -1951,33 +1970,8 @@ void CGroundMoveType::KeepPointingTo(float3 pos, float distance, bool aggressive
 }
 
 void CGroundMoveType::KeepPointingTo(CUnit* unit, float distance, bool aggressive){
-	mainHeadingPos = unit->pos;
-	useMainHeading = aggressive;
-
-	if (useMainHeading
-			&& !owner->weapons.empty()
-			&& (this-owner->weapons[0]->weaponDef->waterweapon
-			|| mainHeadingPos.y >= 0)) {
-
-		float3 dir1 = owner->weapons.front()->mainDir;
-		dir1.y = 0;
-		dir1.Normalize();
-		float3 dir2 = mainHeadingPos - owner->pos;
-		dir2.y = 0;
-		dir2.Normalize();
-
-		if (dir2 != ZeroVector) {
-			short heading =
-				GetHeadingFromVector(dir2.x, dir2.z) -
-				GetHeadingFromVector(dir1.x, dir1.z);
-
-			if (owner->heading != heading
-					&& !(owner->weapons.front()->TryTarget(
-					mainHeadingPos, true, 0))) {
-				progressState = Active;
-			}
-		}
-	}
+	//! wrapper
+	KeepPointingTo(unit->pos, distance, aggressive);
 }
 
 /**
@@ -2028,8 +2022,8 @@ void CGroundMoveType::SetMainHeading(){
 
 void CGroundMoveType::SetMaxSpeed(float speed)
 {
-	maxSpeed        = std::min(speed, owner->unitDef->speed  / GAME_SPEED);
-	maxReverseSpeed = std::min(speed, owner->unitDef->rSpeed / GAME_SPEED);
+	maxSpeed        = std::min(speed, owner->maxSpeed);
+	maxReverseSpeed = std::min(speed, owner->maxReverseSpeed);
 
 	requestedSpeed = speed * 2.0f;
 }
