@@ -3,8 +3,9 @@
 // Author: Mattias "zerver" Radeskog
 // (C) Ware Zerver Tech. http://zerver.net
 // Ware Zerver Tech. licenses this library
-// to be used freely for any purpose, as
-// long as this notice remains unchanged
+// to be used, distributed and modified 
+// freely for any purpose, as long as 
+// this notice remains unchanged
 
 // GML works by "patching" all OpenGL calls. It is injected via a #include "gml.h" statement located in myGL.h.
 // All files that need GL should therefore include myGL.h. INCLUDING gl.h, glu.h, glext.h ... IS FORBIDDEN.
@@ -37,7 +38,7 @@
 #include "gmlcls.h"
 #include "LogOutput.h"
 
-const char *gmlProfMutex = "piece";
+const char *gmlProfMutex = "lua";
 
 #define EXEC_RUN (BYTE *)NULL
 #define EXEC_SYNC (BYTE *)-1
@@ -241,7 +242,8 @@ boost::recursive_mutex grassmutex;
 boost::recursive_mutex &guimutex=selmutex;
 boost::recursive_mutex filemutex;
 boost::recursive_mutex &qnummutex=quadmutex;
-boost::recursive_mutex groupmutex;
+boost::recursive_mutex &groupmutex=selmutex;
+boost::recursive_mutex &grpselmutex=selmutex;
 #endif
 
 // GMLqueue implementation
@@ -976,6 +978,13 @@ inline void QueueHandler(BYTE *&p, BYTE *&ptr) {
 		GML_MAKEHANDLER3(MultiTexCoord2i)
 		GML_MAKEHANDLER3(GetQueryiv)
 		GML_MAKEHANDLER2(GetBooleanv)
+		GML_MAKEHANDLER1(ValidateProgram)
+		GML_MAKEHANDLER3V(Uniform2iv)
+		GML_MAKEHANDLER3V(Uniform3iv)
+		GML_MAKEHANDLER3V(Uniform4iv)
+		GML_MAKEHANDLER3V(Uniform2fv)
+		GML_MAKEHANDLER3V(Uniform3fv)
+		GML_MAKEHANDLER3V(Uniform4fv)
 	}
 }
 
